#include <stdarg.h>
#include <stdio.h>

#include "mk20dx128.h"

#define _BV(n) (1<<(n))

// I2C
#define i2cOUTLen 8
#define i2cINLen 8
volatile uint8_t i2cIN[32],i2cOUT[32],i2cINwIdx,i2cINrIdx,i2cOUTwIdx,i2cOUTrIdx;
#define i2cOUTw(n) if(i2cOUTwIdx < i2cOUTLen) i2cOUT[i2cOUTwIdx++] = n

void i2c0_isr(void);
void portb_isr(void);
uint8_t i2cSDAint = 0;

int main(void) {

	// Onboard LED (Tsy 13)
	PORTC_PCR5  |= _BV(8); 														  // Digital IO PTC5
	GPIOC_PDDR |= _BV(5);																   // Output
	GPIOC_PDOR |= _BV(5);																	 // High

		///////////////////////// I2C


	SIM_SCGC4 |= SIM_SCGC4_I2C0;												 // Enable I2C Clock
	I2C0_C2 = 0x00;
	I2C0_A1	= 0x88;																	   // RX Address
	I2C0_RA = 0x0;											 // Range Address Null = > Only One Addr
	// We'll use PTB0, PTB1 for I2C Slave (Tsy 16 - 17)
	PORTB_PCR0 = PORT_PCR_MUX(2)|PORT_PCR_ODE|PORT_PCR_SRE|PORT_PCR_DSE; 																 // I2C0_SCL
	PORTB_PCR1 = PORT_PCR_MUX(2)|PORT_PCR_ODE|PORT_PCR_SRE|PORT_PCR_DSE; 																 // I2C0_SDA
	I2C0_F = 0x85;																//
	I2C0_FLT = 0x4;																  // Filter Glitches
	I2C0_C1 = I2C_C1_IICEN|I2C_C1_IICIE;
		///////////////////////// Run !
	NVIC_ENABLE_IRQ(IRQ_PORTB);
	NVIC_ENABLE_IRQ(IRQ_I2C0);

	uint8_t i;
	for(i=0;i<i2cOUTLen;i++)
		i2cOUT[i] = i;
	for(i=0;i<i2cINLen;i++)
		i2cIN[i] = ~i;

	uint32_t n;
	while (1) {
		n=0;
		while(n++<0xFFFFF);
		GPIOC_PTOR = _BV(5);
	}
}


void i2c0_isr(void) {
	uint8_t i2cS,i2cC1,c;

	i2cS = I2C0_S;
	i2cC1 = I2C0_C1;
	// This follows the diagram from the datasheet.. Only the slave part

	I2C0_S = I2C_S_IICIF; // clear intr
	if(!(i2cC1 & I2C_C1_MST)) {
		if(i2cS & I2C_S_ARBL) {
				// Arbitration Lost
			I2C0_S |= I2C_S_ARBL;    // clear arbl flag
			if(!(i2cS & I2C_S_IAAS)) {
				return;
			}
		}
		if(i2cS & I2C_S_IAAS) {													 // I2CStart - Kinda
			// If in Slave Rx already, then RepSTART occured, run callback
			if(i2cS & I2C_S_SRW) {								 // Tx Requested - Master wants data
				//I2C0_C1 = I2C_C1_IICEN | I2C_C1_IICIE;
				c = I2C0_D>>1;		  // Read Address, dummy read 'cause only one address recognized
				//I2C0_C1 = I2C_C1_IICEN | I2C_C1_IICIE | I2C_C1_TX;			// Switch to Tx mode
				I2C0_C1 |= I2C_C1_TX;											// Switch to Tx mode
				i2cOUTrIdx=0;
				if(i2cOUTwIdx)
					I2C0_D = i2cOUT[i2cOUTrIdx++];
				else
					I2C0_D = 0x42;
			} else {											 // Rx Requested - Master sends data
				if(!i2cSDAint) { 									   // Use Rising Edge INT on SDA
					PORTB_PCR1 |= PORT_PCR_IRQC(0x09); i2cSDAint = 1;	 // to detect Stop (original
				} 													   // hack from Paul @ pjrc.com)
				// IC20_C1 = //I2C_C1_IICEN | I2C_C1_IICIE; 					// Switch to Rx mode
				I2C0_C1 &= ~I2C_C1_TX;							 				// Switch to Rx mode
				i2cINwIdx = 0;
				c = I2C0_D >> 1;	  // Read Address, dummy read 'cause only one address recognized
			}
		} else {
			if(i2cC1 & I2C_C1_TX) {													 // in TX mode ?
				if(!(i2cS & I2C_S_RXAK)) {								// Master Ack-ed, keep goin'
					//I2C0_C1 = I2C_C1_IICEN | I2C_C1_IICIE | I2C_C1_TX;
					if(i2cOUTrIdx<i2cOUTwIdx)
						I2C0_D = i2cOUT[i2cOUTrIdx++];
					else
						I2C0_D = 0x00;											   // Send Data byte
				} else {												// Master NAcked, stop it...
					// IC20_C1 = //I2C_C1_IICEN | I2C_C1_IICIE; 				// Switch to Rx mode
					I2C0_C1 &= ~I2C_C1_TX;							 			// Switch to Rx mode
					c = I2C0_D;													  // Dummy byte read
				}
			} else {																   // in RX Mode
				//if(!i2cSDAint) { 									   // Use Rising Edge INT on SDA
				//	PORTB_PCR1 |= PORT_PCR_IRQC(0x09); i2cSDAint = 1;	 // to detect Stop (original
				//} 												   // hack from Paul @ pjrc.com)
				if(i2cINwIdx < i2cINLen)
					i2cIN[i2cINwIdx++] = I2C0_D;								   // Read Data byte
				else
					c = I2C0_D;											  // Buffer Full, Dummy read
			}
		}
	}
}

void portb_isr(void) {
	uint32_t isfr = PORTB_ISFR;
	uint8_t i;
	PORTB_ISFR = isfr;
	if((isfr & _BV(1))) {
		if(i2cSDAint && !(I2C0_S & I2C_S_BUSY)) {								// I2C Stop detected
			PORTB_PCR1 &= ~PORT_PCR_IRQC(0x09);	i2cSDAint = 0;						  // Disable INT
			// Read Rxed Data, prepare Tx if needed, ...
			i2cOUTwIdx = 0;
			for(i=0;i<i2cINwIdx;i++)
				i2cOUTw(i2cIN[i]);
		}
	}
}
